﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Calls;
using Hotline.Orders;
using Hotline.Realtimes;
using Hotline.Share.Dtos.TrCallCenter;
using MapsterMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using NewRock.Sdk.Events;
using Org.BouncyCastle.Utilities;
using Serilog.Core;
using SqlSugar;
using StackExchange.Redis;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tr.Sdk;
using Tr.Sdk.Tels;
using XF.Domain.Constants;
using XF.Domain.Repository;

namespace Hotline.Application.Bigscreen
{
	public class SeatStateDataRefreshService : BackgroundService
	{
		private readonly IServiceScopeFactory _serviceScopeFactory;
		private DateTime _lastModifyTime;

		public SeatStateDataRefreshService(IServiceScopeFactory serviceScopeFactory)
		{
			_serviceScopeFactory = serviceScopeFactory;
			_lastModifyTime = DateTime.Now;
		}

		protected override async Task ExecuteAsync(CancellationToken stoppingToken)
		{
			using var scope = _serviceScopeFactory.CreateScope();
			var realtimeService = scope.ServiceProvider.GetRequiredService<IRealtimeService>();
			var logger = scope.ServiceProvider.GetRequiredService<ILogger<BigscreenDataShowRefreshService>>();
			var systemSettingCacheManager = scope.ServiceProvider.GetRequiredService<ISystemSettingCacheManager>();
			var callRepository = scope.ServiceProvider.GetRequiredService<IRepository<TrCallRecord>>();

			try
			{
				int times = int.Parse(systemSettingCacheManager.GetSetting(SettingConstants.BsDataStateChangedTimes)?.SettingValue[0]);
				while (!stoppingToken.IsCancellationRequested)
				{
					//var now = DateTime.Now;
					//var secRange = (now - _lastModifyTime).TotalSeconds;
					//if (secRange >= 10)
					//{
					var call24 = await callRepository.Queryable()
						.Where(x => x.CreatedTime.ToString("yyyy-MM-dd") == DateTime.Now.ToString("yyyy-MM-dd") && x.CallOrderType == Share.Enums.CallCenter.ECallOrderType.Order)
						.Select(x => new { time = x.CreatedTime.ToString("hh"), x.CallDirection }).MergeTable()
						.GroupBy(x => x.time)
						.Select(x => new
						{
							Time = x.time.ToString() + ":00",
							In = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In, 1, 0)),
							Out = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.Out, 1, 0)),
						}).MergeTable().ToListAsync(stoppingToken);
					//await realtimeService.BsSeatStateDataShowChangedAsync(call24, stoppingToken);

					var callTop10 = await callRepository.Queryable()
						.Where(x => x.CreatedTime.ToString("yyyy-MM-dd") == DateTime.Now.ToString("yyyy-MM-dd") && x.CallOrderType == Share.Enums.CallCenter.ECallOrderType.Order)
						.GroupBy(x => x.UserName)
						.Select(x => new
						{
							UserName = x.UserName,
							In = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In, 1, 0))
						}).MergeTable().OrderByDescending(x => x.In).Take(10).ToListAsync(stoppingToken);
					//await realtimeService.BsSeatStateDataShowChangedAsync(callTop10, stoppingToken);

					var callList = await callRepository.Queryable()
						.Where(x => x.CreatedTime.ToString("yyyy-MM-dd") == DateTime.Now.ToString("yyyy-MM-dd") && x.CallOrderType == Share.Enums.CallCenter.ECallOrderType.Order)
						.Select(x => new
						{
							InOn = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && x.OnState == Share.Enums.CallCenter.EOnState.On, 1, 0)),
							ValidOn = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && x.OnState == Share.Enums.CallCenter.EOnState.On && x.Duration > 5, 1, 0)),
							InNoOn = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && x.OnState == Share.Enums.CallCenter.EOnState.NoOn, 1, 0)),
							OutOn = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.Out && x.OnState == Share.Enums.CallCenter.EOnState.On, 1, 0)),
							InQueueNoOn = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && x.OnState == Share.Enums.CallCenter.EOnState.NoOn && x.QueueTims > 0, 1, 0)),
						}).MergeTable().ToListAsync(stoppingToken);
					//await realtimeService.BsSeatStateDataShowChangedAsync(callList, stoppingToken);

					var callAverage = await callRepository.Queryable()
						.Where(x => x.CreatedTime.ToString("yyyy-MM-dd") == DateTime.Now.ToString("yyyy-MM-dd") && x.CallOrderType == Share.Enums.CallCenter.ECallOrderType.Order)
						.Select(x => new { time = x.CreatedTime.ToString("hh"), x.CallDirection }).MergeTable()
						.GroupBy(x => x.time)
						.Select(x => new
						{
							Time = x.time.ToString() + ":00",
							In = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.In, 1, 0)) / 60,
							Out = SqlFunc.AggregateSum(SqlFunc.IIF(x.CallDirection == Share.Enums.CallCenter.ECallDirection.Out, 1, 0)) / 60,
						}).MergeTable().ToListAsync(stoppingToken);
					await realtimeService.BsSeatStateDataShowChangedAsync(new { Call24 = call24, CallTop10 = callTop10, CallList= callList, CallAverage = callAverage }, stoppingToken);

					//logger.LogInformation($"send msg: {orders.First().Address}");
					_lastModifyTime = DateTime.Now;
					//}
					await Task.Delay(times, stoppingToken);
				}
			}
			catch (Exception e)
			{
				logger.LogInformation($"send msg: {e.ToString()}");
				throw;
			}
		}
	}
}
